<?php

namespace Drupal\payment_mercadopago\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsRefundsInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_price\Price;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;  
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use MercadoPago\MercadoPagoConfig;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "mp_offsite_redirect",  
 *   label = "Mercado Pago",
 *   display_label = "Mercado Pago",
 *   forms = {
 *     "offsite-payment" = "Drupal\payment_mercadopago\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
 *     "refund-payment" = "Drupal\payment_mercadopago\PluginForm\PaymentRefundForm\MercadoPagoRefundForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "dinersclub", "discover", "jcb", "maestro", "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class OffsiteRedirect extends OffsitePaymentGatewayBase implements SupportsRefundsInterface {
 
  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'public_key' => '',
      'access_token' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

        $form['public_key'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Public Key'),
          '#default_value' => $this->configuration['public_key'],
          '#required' => TRUE,
      ];

      $form['access_token'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Access Token'),
          '#default_value' => $this->configuration['access_token'],
          '#required' => TRUE,
      ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['public_key'] = $values['public_key'];
      $this->configuration['access_token'] = $values['access_token'];

      // Aquí almacenamos las claves de acceso
      $form['#attached']['drupalSettings']['payment_mercadopago']['public_key'] = $this->configuration['public_key'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    // @todo Add examples of request validation.
    // Note: Since requires_billing_information is FALSE, the order is
    // not guaranteed to have a billing profile. Confirm that
    // $order->getBillingProfile() is not NULL before trying to use it.
    $query_data = $request->query->all();
    $status = $query_data['collection_status'];

    switch ($status) {
      case 'approved':
        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $payment = $payment_storage->create([
          'state' => 'completed', 
          'amount' => $order->getBalance(),
          'payment_gateway' => $this->parentEntity->id(),
          'order_id' => $order->id(),
          'remote_id' => $query_data['merchant_order_id'],
          'remote_state' => $status,
          'authorized' => $this->time->getRequestTime(),
          'completed' => $this->time->getRequestTime(),
        ]);
    
        $payment->save();
        
        break;
      case 'in_process':
        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $payment = $payment_storage->create([
          'state' => 'authorization', 
          'amount' => $order->getBalance(),
          'payment_gateway' => $this->parentEntity->id(),
          'order_id' => $order->id(),
          'remote_id' => $query_data['merchant_order_id'],
          'remote_state' => $status,
          'authorized' => $this->time->getRequestTime(),
          'completed' => $this->time->getRequestTime(),
        ]);

        $payment->save();

        $this->messenger()->addMessage(t('Pago pendiente de aprobación. Procesaremos su pedido una vez que Mercado Pago confirme el pago.', [
        ]));
        break;
      default:
        $this->messenger()->addError(t('Security Error. Illegal access detected.'));
        break;
    }
    
  }

  /**
   * {@inheritdoc}
   */
  public function refundPayment(PaymentInterface $payment, Price $amount = null) {

    $id_pago = $payment->getRemoteId(); // Utiliza el método correcto para obtener el ID del pago.
    $access_token = $this->configuration['access_token']; // Obtén el token de acceso desde la configuración.

    if (!$id_pago || !$access_token) {
      // Maneja las condiciones de error.
      return;
    }

    $client = new Client();
    $url = "https://api.mercadopago.com/v1/payments/{$id_pago}/refunds";

    $this->assertPaymentState($payment, ['completed', 'partially_refunded']);
    // If not specified, refund the entire amount.
    $amountPayment = $amount ?: $payment->getAmount();
    $this->assertRefundAmount($payment, $amountPayment);

    $old_refunded_amount = $payment->getRefundedAmount();
    $new_refunded_amount = $old_refunded_amount->add($amountPayment);
    try {
      if ($new_refunded_amount->lessThan($payment->getAmount())) {
        $payment->state = 'partially_refunded';
        $response = $client->post($url, [
          'headers' => [
            'Authorization' => "Bearer {$access_token}",
            'Content-Type' => 'application/json',
            'X-Idempotency-Key' => uniqid(), // Genera una clave única para la idempotencia.
          ],
          'json' => [
            'amount' => $amount ? $amount->getNumber() : $payment->getAmount()->getNumber(), // Utiliza el monto especificado o el monto original del pago.
          ],
        ]);
      }
      else {
        $payment->state = 'refunded';
        $response = $client->post($url, [
          'headers' => [
            'Authorization' => "Bearer {$access_token}",
            'Content-Type' => 'application/json',
            'X-Idempotency-Key' => uniqid(), // Genera una clave única para la idempotencia.
          ],
        ]);
      }

      if ($response->getStatusCode() === 200) {
        // Reembolso exitoso. Maneja el estado según sea necesario.
        $this->messenger()->addMessage(t('El reembolso se realizó correctamente.'));
        $payment->setRefundedAmount($new_refunded_amount);
        $payment->save();
      } else {
        // Maneja la respuesta de error de la API.
        $response_data = json_decode($response->getBody()->getContents(), TRUE);
        $error_message = isset($response_data['message']) ? $response_data['message'] : t('Error desconocido en el reembolso.');
        $this->messenger()->addError($error_message);
      }
    } catch (RequestException $e) {
      // Maneja la excepción de solicitud de Guzzle.
      $this->messenger()->addError(t('Hubo un error al intentar realizar el reembolso. Contacte con un administrador'));
    }
  }
}

