<?php

namespace Drupal\payment_mercadopago\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\commerce_order\Entity\Order;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Client;

class PaymentOffsiteForm extends BasePaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\payment_mercadopago\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();

    // Obtener la configuración, incluyendo access_token y public_key
    $configuration = $payment_gateway_plugin->getConfiguration();
    $public_key = $configuration['public_key'];  // Aquí obtenemos la clave pública
    $access_token = $configuration['access_token'];  // Aquí obtenemos el access token

    // Configurar URLs de retorno
    $backUrls = [
      'success' => $form['#return_url'],
      'failure' => $form['#cancel_url'],
      'pending' => $form['#return_url'],
    ];

    $order_id = $payment->getOrderId();
    $items = [];

    if ($order_id) {
      $order = Order::load($order_id);
      if ($order) {
        $line_items = $order->getItems();
        foreach ($line_items as $line_item) {
          $quantity = (int) $line_item->getQuantity();  // Convertir a entero
          $items[] = [
            "title" => $line_item->getTitle(),
            "id" => $line_item->getPurchasedEntityId(),
            "description" => $line_item->getTitle(),
            "quantity" => $quantity,  // Asegurarse de que sea un número entero
            "unit_price" => (float) $line_item->getUnitPrice()->getNumber(),  // Convertir el precio a flotante
          ];
        }          
      }
    }

    // Crear preferencia de pago
    $preference_data = [
      "items" => $items,
      "back_urls" => $backUrls,
      "auto_return" => "approved",
      "statement_descriptor" => "NAME_DISPLAYED_IN_USER_BILLING",
      "external_reference" => $order_id,
    ];

    // Enviar la preferencia a la API de MercadoPago usando Guzzle
    $client = new Client();
    try {
      $response = $client->post('https://api.mercadopago.com/checkout/preferences', [
        'headers' => [
          'Authorization' => 'Bearer ' . $access_token,
          'Content-Type' => 'application/json',
        ],
        'json' => $preference_data,
      ]);

      // Obtener la respuesta con la preferencia creada
      $preference_response = json_decode($response->getBody(), TRUE);
      $preference_id = $preference_response['id'];

    } catch (\Exception $e) {
      // Registrar el error detallado en los logs de Drupal
      \Drupal::logger('mercadopago')->error('Error al crear la preferencia de MercadoPago: ' . $e->getMessage());

      // Lanzar una excepción para que Drupal muestre el error al usuario
      throw new PaymentGatewayException('No se pudo crear la preferencia de MercadoPago. Detalles: ' . $e->getMessage());
    }

    // Mensaje que se mostrará al usuario
    $form['content'] = [
      '#type' => 'markup',
      '#markup' => '<div class="container"><h2>Pagar con Mercado Pago</h2>
      <p>Realiza tu pago de forma segura con Mercado Pago. Serás redirigido a Mercado Pago para completar la transacción y luego volverás automáticamente a nuestra tienda.</p>',
    ];

    $form['loading_message'] = [
      '#type' => 'markup',
      '#markup' => '<div id="mercadopago-bricks-contaner__PaymentCard" class="legosss"></div>',
    ];

    $form['loading'] = [
      '#type' => 'markup',
      '#markup' => '<div id="loading" class="text-center d-flex justify-content-center align-items-center"><div class="spinner-border text-dark" role="status">
      <span class="visually-hidden"></span>
    </div><span class="ms-2">Esperando a Mercado Pago ...</span></div></div>',
    ];

    // Adjuntar librerías y configuración
    $form['#attached']['library'][] = 'payment_mercadopago/commerce-mercadopago';
    $form['#attached']['drupalSettings']['payment_mercadopago']['public_key'] = $public_key;  // Agregar clave pública
    $form['#attached']['drupalSettings']['payment_mercadopago']['access_token'] = $access_token;
    $form['#attached']['drupalSettings']['payment_mercadopago']['preference'] = $preference_id;

    return $form;
  }
}
